import asyncio
import concurrent.futures
import functools
import importlib
import importlib.metadata
import types
import typing


__version__: str = importlib.metadata.version("aioimport")


class Importer:
    _executor_factory: typing.Callable[..., concurrent.futures.ThreadPoolExecutor]
    _executor: typing.Optional[concurrent.futures.ThreadPoolExecutor] = None

    def __init__(
        self,
        max_workers: typing.Optional[int] = None,
        thread_name_prefix: str = "",
        initializer: typing.Optional[typing.Callable] = None,
        initargs: typing.Tuple = (),
    ) -> None:
        # See https://github.com/python/mypy/issues/708 for type ignore reason
        self._executor_factory = functools.partial(  # type: ignore
            concurrent.futures.ThreadPoolExecutor,
            max_workers=max_workers,
            thread_name_prefix=thread_name_prefix,
            initializer=initializer,
            initargs=initargs,
        )

    def _get_executor(self) -> concurrent.futures.ThreadPoolExecutor:
        if self._executor is None:
            self._executor = self._executor_factory()
        return self._executor

    def shutdown(self, wait: bool = True) -> None:
        if self._executor is not None:
            self._executor.shutdown(wait=wait)
            self._executor = None

    async def import_module(
        self, name: str, package: typing.Optional[str] = None
    ) -> types.ModuleType:
        executor = self._get_executor()
        return await asyncio.get_event_loop().run_in_executor(
            executor, functools.partial(importlib.import_module, name, package=package)
        )

    async def cache_module(
        self, name: str, package: typing.Optional[str] = None
    ) -> None:
        await self.import_module(name, package=package)


default = Importer()


async def import_module(
    name: str, package: typing.Optional[str] = None
) -> types.ModuleType:
    return await default.import_module(name, package=package)


async def cache_module(name: str, package: typing.Optional[str] = None) -> None:
    await default.cache_module(name, package=package)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aioimport']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'aioimport',
    'version': '0.1.1',
    'description': 'Asynchronous module import for asyncio',
    'long_description': '# aioimport\nAsynchronous module import for asyncio\n\n## Getting Started\n\n### Installing\n\nInstall from [PyPI](https://pypi.org/project/aioimport/) using:\n\n```\npip install aioimport\n```\n\n### The problem\n\nSome nasty modules have long running operations during import\n\n### The naive solution\n\nFirst thing that comes to mind is to postpone import by moving it into the function that need that module:\n\n```python\nasync def my_work(self) -> None:\n    import this  # will block until imported\n```\n\nIt reduces your startup time, but it is still blocking your event loop.\n\nThat\'s where `aioimport` comes in.\n\n### Usage\n\nThe preferred way to use `aioimport` is:\n```python\nimport aioimport\n\nasync def my_work(self) -> None:\n    await aioimport.cache_module("this")  # will asynchronously import module\n    import this  # will be instantaneous since `this` is already in import cache \n```\n\nAlso `aioimport` can be also used the following way:\n```python\nimport aioimport\n\nasync def my_work(self) -> None:\n    this = await aioimport.import_module("this")  # will asynchronously import module\n    # now `this` has exactly the same module as you would have gotten be doing `import this`\n```\nBut your types will be all messed up.\n\n### How it works\n\nModule import is done in thread executor.\n\nBe aware of the fact that GIL still exists and technically import is done concurrently rather than in parallel with your code.\n\n## License\n\nThis project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details\n',
    'author': 'rndr',
    'author_email': 'rndr@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rndr/aioimport',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
